import sys
from .fontMods import TimeFonts
from .mylogger import logger
from time import sleep
from datetime import datetime as dt



class TimeGenerator:
    nums = ["zero", "one", "two", "three", "four",
            "five", "six", "seven", "eight", "nine",
            "ten", "eleven", "twelve", "thirteen",
            "fourteen", "fifteen", "sixteen", 
            "seventeen", "eighteen", "nineteen", 
            "twenty", "twenty one", "twenty two", 
            "twenty three", "twenty four", 
            "twenty five", "twenty six", "twenty seven",
            "twenty eight", "twenty nine"]

    def __init__(self,case='lower',format='12'):
        self.case = case
        self.format = format

    def get_words_from_time(self,h=None,m=None):
        """
        Returns a sentence giving the time based on hour and minute

        Parameter
        ---------
        h: hour as integer 12 or 24-hour format
        m: minute as integer 0<=m<60
        """
        if h is None:
            h = self.get_current_hour()
        if m is None:
            m = self.get_approximate_minute()
        
        if self.format=='12':
            if h > 12: h = h - 12

        if   m==0:  time_sentence = f"{self.nums[h]} o'clock"
        elif m==1:  time_sentence = f"one minute past {self.nums[h]}"
        elif m==59: time_sentence = f"one minute to {self.nums[(h % 12) + 1]}"
        elif m==15: time_sentence = f"quarter past {self.nums[h]}"
        elif m==30: time_sentence = f"half past {self.nums[h]}"
        elif m==45: time_sentence = f"quarter to {self.nums[h]}"
        elif m<=30: time_sentence = f"{self.nums[m]} minutes past {self.nums[h]}"
        elif m>=30: time_sentence = f"{self.nums[60 - m]} minutes to {self.nums[(h % 12) + 1]}"

        if self.case != 'lower':
            return ("it is "+time_sentence).upper()
        
        return ("it is "+time_sentence)

    def get_current_hour(self):
        return int(dt.now().hour)

    def get_approximate_minute(self):
        return int(dt.now().minute/5)*5

    def get_current_minute(self):
        return int(dt.now().minute)

    def print_time(self):
        print(TimeFonts.BOLD + self.get_words_from_time() + TimeFonts.END)

    def print_time_matrix(self):
        matrix = TimeFonts(time_sentence=self.get_words_from_time())
        matrix.show()


from .clockFace import TimeGenerator
from .mylogger import logger
import click

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

@click.command(context_settings=CONTEXT_SETTINGS)
@click.option('--show','-s',is_flag=True, help="Show the current time")
@click.option('--matrix','-m',is_flag=True, help="Show time as matrix")
@click.option('--debug','-d',is_flag=True,help="Run in debug mode")
def main(show,matrix,debug):
    if debug:
        logger.setLevel('DEBUG')
    if show:
        
        if matrix:
            logger.debug('Getting current time as matrix')
            TimeGenerator('upper').print_time_matrix()
        else:
            logger.debug('Getting current time as sentence')
            TimeGenerator('upper').print_time()

if __name__ == '__main__':
    main()


from .mylogger import logger

# 0   ITLISASTHTEN
# 1   ACFIFTEENDCO
# 2   TWENTYXFIVEW
# 3   THIRTYFTENOS
# 4   MINUTESETOUR
# 5   PASTORUFOURT
# 6   SEVENXTWELVE
# 7   NINEDIVECTWO
# 8   EIGHTFELEVEN
# 9   SIXTHREEONEG
# 10  TENSEZO'CLOCK

class TimeFonts:

    PURPLE      = '\033[95m'
    CYAN        = '\033[96m'
    DARKCYAN    = '\033[36m'
    BLUE        = '\033[94m'
    GREEN       = '\033[92m'
    YELLOW      = '\033[93m'
    RED         = '\033[91m'
    BOLD        = '\033[1m'
    UNDERLINE   = '\033[4m'
    END         = '\033[0m'    

    #          0   1   2   3   4   5   6    7   8   9   10  11  12
    line0 =  ["I","T","L","I","S","A","S", "T","H","T","E","N"," "]
    line1 =  ["A","C","F","I","F","T","E", "E","N","D","C","O"," "]
    line2 =  ["T","W","E","N","T","Y","X", "F","I","V","E","W"," "]
    line3 =  ["T","H","I","R","T","Y","F", "T","E","N","O","S"," "]
    line4 =  ["M","I","N","U","T","E","S", "E","T","O","U","R"," "]
    line5 =  ["P","A","S","T","O","R","U", "F","O","U","R","T"," "]
    line6 =  ["S","E","V","E","N","X","T", "W","E","L","V","E"," "]
    line7 =  ["N","I","N","E","D","I","V", "E","C","T","W","O"," "]
    line8 =  ["E","I","G","H","T","F","E", "L","E","V","E","N"," "]
    line9 =  ["S","I","X","T","H","R","E", "E","O","N","E","G"," "]
    line10 = ["T","E","N","S","E","Z","O'","C","L","O","C","K"," "]

    all_lines = [line0,line1,line2,line3,line4,line5,line6,line7,line8,line9,line10,]
    
    time_key_maps = {
        'it'        : [0,   0,  2],
        'is'        : [0,   3,  5],
        'one'       : [9,   8,  11],
        'two'       : [7,   9,  -1],
        'three'     : [9,   3,  8],
        'four'      : [5,   7,  11],
        'five'      : [2,   7,  11],
        'six'       : [9,   0,  3],
        'seven'     : [6,   0,  5],
        'nine'      : [7,   0,  4],
        'ten'       : [0,   9,  12],
        'TEN'       : [10,  0,  3],
        'eleven'    : [8,   6,  12],
        'twelve'    : [6,   6,  12],
        'fifteen'   : [1,   2,  9],
        'quarter'   : [1,   2,  9],
        'twenty'    : [2,   0,  6],
        'thirty'    : [3,   0,  6],
        'half'      : [3,   0,  6],
        'minutes'   : [4,   0,  7],
        'past'      : [5,   0,  4],
        'to'        : [4,   8,  10],
        "o'clock"   : [10,  6,  12]

        
    }

    def __init__(self,time_sentence) -> None:
        self.time_sentence = time_sentence
        # self.time_sentence = "it is ten minutes to twelve"
        # self.time_sentence = "it is ten o'clock"
        # self.time_sentence = "it is five minutes past zero"
        # self.time_sentence = "it is zero o'clock"
        self.get_word_locations()

    def get_word_locations(self):
        logger.debug(f'Showing matrix for "{self.time_sentence}"')
        
        # from the time as sentence, take each word
        logger.debug(self.time_sentence.split(' '))
        self.word_locations = []
        for each_word in self.time_sentence.split(' '):


            # corner case
            if each_word == 'quarter': each_word = 'fifteen'
            if each_word == 'half': each_word = 'thirty'
            if each_word == 'zero': each_word = 'twelve'

            logger.debug(f'Checking word "{each_word}" in one {["".join(s) for s in self.all_lines]}')
            

            # check that word in each line
            for each_line in self.all_lines:

                if each_word.lower() in ''.join(each_line).lower():

                    logger.debug(f'MATCH found for "{each_word.lower()}" in {"".join(each_line).lower()}')
                    word_location = self.time_key_maps.get(each_word.lower())

                    if word_location is not None:
                        logger.debug(f'Appending location of "{each_word.lower()}":{word_location}')
                        self.word_locations.append(word_location)

                    # if match is found break
                    break
            logger.debug(f'--------------------------------------')
    
    def clean_locations(self,locations):

        logger.debug(f'Attempting to clean {locations}')
        # Group the inner lists by their first element using a dictionary
        temp_dict = {}
        for inner_list in locations:
            key = inner_list[0]
            value = inner_list[1:]
            if key not in temp_dict:
                temp_dict[key] = []
            temp_dict[key].append(value)

        # Convert the dictionary to the desired format
        new_list = []
        for key, value in temp_dict.items():
            new_list.append([key] + value)

        # remove duplicates
        new_list = [[x[0]] + list(set(map(tuple, x[1:]))) for x in new_list]

        #sort the words
        new_list = [[x[0]] + sorted(x[1:], key=lambda y: y[0]) for x in new_list]
        logger.debug(f'Cleaned locations {new_list}')
        return new_list


    def show(self):
        
        # self.word_locations = [[0, 0, 2], [0, 3, 5], [0,   9,  12], [10, 0, 3], [4, 0, 7], [4, 8, 10], [10, 0, 3]]
        # self.word_locations = [[0, 0, 2], [0, 3, 5], [3,   0,  6],[5, 0, 4], [8, 6, 12]]
        # [[0, [0, 2], [3, 5]], [10, [0, 3], [0, 3]], [4, [0, 7], [8, 10]]]
        locations = self.clean_locations(self.word_locations)

        for line_no,line in enumerate(self.all_lines):
            # logger.debug(f'line #{line_no}: {line}')
            if line_no not in [s[0] for s in locations]:
                logger.debug(f'{line_no} not required {locations}')
                print(" ".join(line))
            else:
                for word_loc in locations:
                    if word_loc[0] == line_no:
                        logger.debug(f'{word_loc}')
                        words = word_loc[1:]
                        if len(words)==2:
                            logger.debug(f'Two words {words}')
                            print(self.CYAN + self.BOLD + 
                                    " ".join(line[words[0][0]:words[0][1]]) + self.END + " " +  
                                    " ".join(line[words[0][1]:words[1][0]]) + " " + 
                                    self.CYAN + self.BOLD + 
                                    " ".join(line[words[1][0]:words[1][1]]) + " " +  self.END + 
                                    " ".join(line[words[1][1]:])
                                    )
                        elif len(words)==3:
                            logger.debug(f'Three words {words}')
                            print(self.CYAN + self.BOLD + 
                                    " ".join(line[words[0][0]:words[0][1]]) + self.END + " " +  
                                    " ".join(line[words[0][1]:words[1][0]]) + " " + 
                                    self.CYAN + self.BOLD + 
                                    " ".join(line[words[1][0]:words[1][1]]) + " " +  self.END + 
                                    " ".join(line[words[1][1]:words[2][0]]) + " " +
                                    self.CYAN + self.BOLD +
                                    " ".join(line[words[2][0]:words[2][1]]) +                                 
                                     self.END )
                        else:
                            # if the word is beginning after few positions
                            if words[0][0] != 0:
                                print(" ".join(line[0:words[0][0]]),end=' ')

                            print(self.CYAN + self.BOLD +
                                    " ".join(line[words[0][0]:words[0][1]]) + 
                                    " " +  
                                    self.END + 
                                    " ".join(line[words[0][1]:])
                                    )

        

                        
            


from unittest import TestCase, TextTestRunner, defaultTestLoader
from source.clockFace import TimeGenerator
from datetime import  datetime

class TestClockFace(TestCase):

    @classmethod
    def setUpClass(self):
        print("Test Start")
        self.timegen = TimeGenerator()

    @classmethod
    def tearDownClass(self):
        print("Test Done")

    def test_time_oclock(self):
        time = self.timegen.get_words_from_time(h=1,m=00)
        self.assertEqual(time,"it is one o'clock")

    def test_time_upper_case(self):
        self.timegen = TimeGenerator(case='upper',format='24')
        time = self.timegen.get_words_from_time(h=18,m=30)
        self.assertEqual(time,"IT IS HALF PAST EIGHTEEN")

    def test_half_past_one(self):
        time = self.timegen.get_words_from_time(h=1,m=30)
        self.assertEqual(time,"it is half past one")

    def test_quarter_to_13_format_24(self):
        self.timegen = TimeGenerator(case='lower',format='24')
        time = self.timegen.get_words_from_time(h=13,m=45)
        self.assertEqual(time,"it is quarter to thirteen")

    def test_quarter_to_13_format_12(self):
        self.timegen = TimeGenerator(case='lower',format='12')
        time = self.timegen.get_words_from_time(h=13,m=45)
        self.assertEqual(time,"it is quarter to one")

    def test_quarter_to_13_format_24_upper(self):
        self.timegen = TimeGenerator(case='upper',format='24')
        time = self.timegen.get_words_from_time(h=13,m=45)
        self.assertEqual(time,"IT IS QUARTER TO THIRTEEN")

    def test_quarter_to_13_format_12_upper(self):
        self.timegen = TimeGenerator(case='upper',format='12')
        time = self.timegen.get_words_from_time(h=13,m=45)
        self.assertEqual(time,"IT IS QUARTER TO ONE")

    def test_get_current_hour(self):
        current_hour = datetime.now().hour
        self.assertEqual(self.timegen.get_current_hour(),current_hour)

    def test_fail_current_minute_with_approx_minute(self):
        current_minute = datetime.now().minute

        # if the current minute is like 10,20,.. then approximate = current minute
        if current_minute % 5 == 0:
            self.assertEqual(self.timegen.get_approximate_minute(),current_minute)
        else:
            self.assertNotEqual(self.timegen.get_approximate_minute(),current_minute)
    
    def test_random_time(self):
        time = self.timegen.get_words_from_time(h=None,m=None)
        assumed_time = "half past two"

        # Handling a corner case:
        # when the current_time and assumed_time is same,
        # happens if the test runs exactly at half past two
        system_time = datetime.now()
        if system_time.minute == 30 and system_time.hour == 2:
            self.assertEqual(time,assumed_time)
        else:
            self.assertNotEqual(time,assumed_time)

def run():
    TextTestRunner().run(defaultTestLoader.loadTestsFromTestCase(TestClockFace))

if __name__ == '__main__':

    run()



from setuptools import setup, find_packages
import io
from os import path
requirements = [l.strip() for l in open('requirements.txt').readlines()]

# Get the long description from the README file

here = path.abspath(path.dirname(__file__))

with io.open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# This gets deployed when a new release is made by github actions
VERSION = '0.1.4'

setup(
    name='py-text-clock',
    url='https://github.com/manojmanivannan/py-clock',
    version=VERSION,
    author='Manoj Manivannan',
    author_email='manojm18@live.in',
    description='A verbose Clock',
    long_description=long_description,
    packages=find_packages(),
    entry_points={
    'console_scripts': ['py-clock=source.command_line:main'],
    },
    install_requires=requirements,
    setup_requires=['wheel','setuptools'],
    include_package_data=True,
)
